@@ -26,6 +26,7 @@ geary.intern <- function(x, listw, n, zero.policy, type="geary") {
 
 geary.test <- function(x, listw, randomisation=TRUE, zero.policy=FALSE,
     alternative="less", spChk=NULL) {
+	alternative <- match.arg(alternative, c("less", "greater", "two.sided"))
 	if(!inherits(listw, "listw")) stop(paste(deparse(substitute(listw)),
 		"is not a listw object"))
 	if(!is.numeric(x)) stop(paste(deparse(substitute(x)),
@@ -36,8 +37,7 @@ geary.test <- function(x, listw, randomisation=TRUE, zero.policy=FALSE,
 	if (is.null(spChk)) spChk <- get.spChkOption()
 	if (spChk && !chkIDs(x, listw))
 		stop("Check of data and weights ID integrity failed")
-	if (!(alternative %in% c("greater", "less", "two.sided")))
-		stop("alternative must be one of: \"greater\", \"less\", or \"two.sided\"")
+	
 	wc <- spweights.constants(listw, zero.policy)
 	S02 <- wc$S0*wc$S0
 	res <- geary(x, listw, wc$n, wc$n1, wc$S0, zero.policy)
@@ -76,6 +76,7 @@ geary.test <- function(x, listw, randomisation=TRUE, zero.policy=FALSE,
 
 geary.mc <- function(x, listw, nsim, zero.policy=FALSE,
 	alternative="less", spChk=NULL) {
+	alternative <- match.arg(alternative, c("less", "greater"))
 	if(!inherits(listw, "listw")) stop(paste(deparse(substitute(listw)),
 		"is not a listw object"))
 	if(!is.numeric(x)) stop(paste(deparse(substitute(x)),
@@ -88,8 +89,6 @@ geary.mc <- function(x, listw, nsim, zero.policy=FALSE,
 	if (spChk && !chkIDs(x, listw))
 		stop("Check of data and weights ID integrity failed")
 	if(nsim > gamma(n+1)) stop("nsim too large for this n")
-	if (!(alternative %in% c("greater", "less", "two.sided")))
-		stop("alternative must be one of: \"greater\", \"less\", or \"two.sided\"")
 	wc <- spweights.constants(listw, zero.policy)
 	res <- numeric(length=nsim+1)
 	for (i in 1:nsim) res[i] <- geary(sample(x), listw, n, wc$n1, wc$S0,
@@ -99,9 +98,10 @@ geary.mc <- function(x, listw, nsim, zero.policy=FALSE,
 	xrank <- rankres[length(res)]
 	diff <- nsim - xrank
 	diff <- ifelse(diff > 0, diff, 0)
-        pval <- (diff + 1)/(nsim+1)
-	if (alternative == "less") pval <- 1 - pval
-	else if (alternative == "two.sided") pval <- 2 * pval
+        if (alternative == "less") 
+        	pval <- punif((diff + 1)/(nsim + 1), lower.tail=FALSE)
+    	else if (alternative == "greater") 
+        	pval <- punif((diff + 1)/(nsim + 1))
 	statistic <- res[nsim+1]
 	names(statistic) <- "statistic"
 	parameter <- xrank
